@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2010 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -52,14 +52,21 @@
 #include <config.h>
 #endif
 
+#include <gmime/gmime.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <fcntl.h>
 #include <errno.h>
+#ifdef G_OS_WIN32
+#include <io.h>
+#endif
 
 #ifdef HAVE_GETOPT_H
 #define _GNU_SOURCE
@@ -68,8 +75,6 @@
 #include "getopt.h"
 #endif
 
-#include <gmime/gmime.h>
-
 
 static struct option longopts[] = {
 	{ "help",        no_argument,       NULL, 'h' },
@@ -131,14 +136,6 @@ uuencode (const char *progname, int argc, char **argv)
 		return -1;
 	}
 	
-	if ((fd = dup (1)) == -1) {
-		fprintf (stderr, "%s: cannot open stdout: %s\n",
-			 progname, strerror (errno));
-		return -1;
-	}
-	
-	ostream = g_mime_stream_fs_new (fd);
-	
 	if (optind + 1 < argc)
 		filename = argv[optind++];
 	else
@@ -146,39 +143,41 @@ uuencode (const char *progname, int argc, char **argv)
 	
 	name = argv[optind];
 	
+	/* open our input file... */
 	if ((fd = filename ? open (filename, O_RDONLY) : dup (0)) == -1) {
 		fprintf (stderr, "%s: %s: %s\n", progname,
 			 filename ? filename : "stdin",
-			 strerror (errno));
-		g_object_unref (ostream);
+			 g_strerror (errno));
 		return -1;
 	}
 	
+	/* stat() our input file for file mode permissions */
 	if (fstat (fd, &st) == -1) {
 		fprintf (stderr, "%s: %s: %s\n", progname,
 			 filename ? filename : "stdin",
-			 strerror (errno));
-		g_object_unref (ostream);
+			 g_strerror (errno));
+		close (fd);
 		return -1;
 	}
 	
-	if (g_mime_stream_printf (ostream, "begin%s %.3o %s\n",
-				  base64 ? "-base64" : "", st.st_mode & 0777, name) == -1) {
-		fprintf (stderr, "%s: %s\n", progname, strerror (errno));
-		g_object_unref (ostream);
-		return -1;
-	}
+	printf ("begin%s %.3o %s\n", base64 ? "-base64" : "", st.st_mode & 0777, name);
+	fflush (stdout);
 	
 	istream = g_mime_stream_fs_new (fd);
 	
+	/* open our output stream */
+	ostream = g_mime_stream_fs_new (1);
+	g_mime_stream_fs_set_owner ((GMimeStreamFs *) ostream, FALSE);
+	
 	fstream = g_mime_stream_filter_new (ostream);
 	
+	/* attach an encode filter */
 	filter = g_mime_filter_basic_new (encoding, TRUE);
 	g_mime_stream_filter_add ((GMimeStreamFilter *) fstream, filter);
 	g_object_unref (filter);
 	
 	if (g_mime_stream_write_to_stream (istream, fstream) == -1) {
-		fprintf (stderr, "%s: %s\n", progname, strerror (errno));
+		fprintf (stderr, "%s: %s\n", progname, g_strerror (errno));
 		g_object_unref (fstream);
 		g_object_unref (istream);
 		g_object_unref (ostream);
@@ -190,7 +189,7 @@ uuencode (const char *progname, int argc, char **argv)
 	g_object_unref (istream);
 	
 	if (g_mime_stream_write_string (ostream, base64 ? "====\n" : "end\n") == -1) {
-		fprintf (stderr, "%s: %s\n", progname, strerror (errno));
+		fprintf (stderr, "%s: %s\n", progname, g_strerror (errno));
 		g_object_unref (ostream);
 		return -1;
 	}
